/*
 * testhash.c: test program for the longhash module.
 *
 *	       specifically, let's have a hash : string->long
 *
 * (C) Duncan C. White, 1996-2017 although it seems longer:-)
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "longhash.h"



void myPrint( FILE *out, longhashkey k, longhashvalue v )
{
	fprintf( out, "%s: %ld\n", k, v );
}


static longhash h1;


void test( longhashkey k, int expected )
{
	long v = longhashFind( h1, k );
	if( v != -1 )
	{
		if( ! expected )
		{
			printf( "error: '%s' found - ", k );
		}
		myPrint( stdout, k, v );
	} else
	{
		if( expected )
		{
			printf( "error: '%s' not found\n", k );
		}
	}
}


void lookuptest( int x1, int x2, int x3, int x4, int xaard, int xgrumble )
{
	printf( "lookup tests:\n" );
	test( "one", x1 );
	test( "two", x2 );
	test( "three", x3 );
	test( "four", x4 );
	test( "aardvark", xaard );
	test( "grumble", xgrumble );
}




/* 
 * each_cb: print out a (k,v) pair to the FILE *
 *	    hidden in the arg; used in the main
 *	    program to print the hash ourselves.
 */

static void each_cb( longhashkey k, longhashvalue v, void * arg )
{
	FILE *out = (FILE *)arg;
	myPrint( out, k, v );
}


int main( int argc, char **argv )
{
	h1 = longhashCreate( myPrint );

	if( argc > 1 )
	{
		malloc(strlen(argv[1]));
	}

	longhashSet( h1, "one", 1 );
	longhashSet( h1, "two", 2 );
	longhashSet( h1, "three", 3 );
	longhashSet( h1, "four", 4 );

	printf( "here's the hash:\n" );
	longhashDump( stdout, h1 );

	lookuptest( 1, 1, 1, 1, 0, 0 );

	printf( "modify hash a bit:\n" );
	longhashSet( h1, "one", 3 );
	longhashSet( h1, "aardvark", 17 );
	longhashSet( h1, "three", 30 );
	longhashInc( h1, "three" );
	longhashInc( h1, "four" );
	longhashInc( h1, "five" );

	printf( "now here's the hash:\n" );
	longhashDump( stdout, h1 );

	lookuptest( 1, 1, 1, 1, 1, 0 );

	printf( "\nprint the hash out using an explicit foreach loop:\n{\n" );
	longhashForeach( h1, &each_cb, stdout );
	printf( "}\n" );

	printf( "\ncopy the hash\n" );
	longhash h2 = longhashCopy( h1 );

	printf( "modify both hashes a bit:\n" );
	longhashSet( h1, "one", 18 );
	longhashSet( h2, "one", 11 );

	printf( "print h1:\n" );
	longhashDump( stdout, h1 );

	printf( "print h2:\n" );
	longhashDump( stdout, h2 );

	printf( "free the original hash\n" );
	longhashFree( h1 );

	printf( "print the copy again:\n" );
	longhashDump( stdout, h2 );

	printf( "free the copy\n" );
	longhashFree( h2 );

	exit(0);
	return 0;
}
