/*
 * mintesttree: simple test for automatically generated "tree" data type
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "datatypes.h"


/* recursive tree leaf counter
 * In Haskell, this'd be:
 *   nleaves(leaf(name))   = 1
 *   nleaves(node(l,r)) = nleaves(l) + nleaves(r)
 */
int nleaves( tree t )
{
	if( tree_kind(t) == tree_is_leaf )     	// a leaf
	{
		string name; get_tree_leaf( t, &name );
		// leaf( name ): contains 1 leaf
		return 1;
	} else						// a node
	{
		tree l, r; get_tree_node( t, &l, &r );
		// node( l, r ): process l and r trees:
		return nleaves(l) + nleaves(r);
	}
}


int main( void )
{
	tree t1 = tree_leaf( "absolutely" );
	printf( "1-leaf tree is: " );
	print_tree( stdout, t1 );
	printf( "\ntree contains %d leaves\n\n", nleaves(t1) );

	tree t2 = tree_leaf( "fabulous" );
	tree t = tree_node( t1, t2 );

	assert( tree_kind(t) == tree_is_node );

	tree l, r;
	get_tree_node( t, &l, &r );
	assert( tree_kind(l) == tree_is_leaf );
	assert( tree_kind(r) == tree_is_leaf );

	assert( nleaves(t) == 2 );

	printf( "2-leaf tree is: " );
	print_tree( stdout, t );
	printf( "\ntree contains %d leaves\n\n", nleaves(t) );

	tree t3 = tree_leaf( "patty" );
	t = tree_node( t, t3 );
	assert( tree_kind(t) == tree_is_node );
	assert( nleaves(t) == 3 );

	printf( "3-leaf tree is: " );
	print_tree( stdout, t );
	printf( "\ntree contains %d leaves\n\n", nleaves(t) );

	//free_tree( t );
	return(0);
}
